/* $Id: thishost.c,v 1.3 2000/10/10 22:10:44 dps Exp $ */
/* Check if the givne ip numbers are this machine */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */
#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif /* HAVE_STDIO_H */
#ifdef HACVE_CTYPE_H
#include <ctype.h>
#endif /* HAVE_CTYPE_H */
#ifdef HAVE_TIME_H
#include <time.h>
#endif /* HAVE_TIME_H */
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif /* HAVE_SYS_TIME_H */
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */
#ifdef HAVE_ALLOCA_H
#include <alloca.h>
#endif /* HAVE_ALLOCA_H */
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif /* HAVE_ERRNO_H */
#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif /* HAVE_NETDB_H */
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif /* HAVE_SYS_TYPES_H */
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif /* HAVE_SYS_SOCKET_H */
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif /* HAVE_NETINET_IN_H */
#ifdef HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif /* HAVE_ARPA_INET_H */
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */

#ifndef PORT_RANGE
#define PORT_RANGE 40
#endif /* PORT_RANGE */


static int recv_socket(unsigned int port, int *fd)
{
    struct sockaddr_in name;
    int sock, b, ports;

    *fd=-1;
    if ((sock=socket(PF_INET, SOCK_DGRAM, 0))<0)
	return -1;

    name.sin_family=AF_INET;
    name.sin_addr.s_addr=htonl(INADDR_ANY);
    ports=PORT_RANGE;
    do
    {
	name.sin_port=htons(port++);
	ports--;
	b=bind(sock, (struct sockaddr *) &name, sizeof(name));
    } while (b<0 && ports>0);
    if (b<0)
    {
	close(sock);
	return -1;
    }
    *fd=sock;
    return port-1;
}

static int send_dgram(const struct in_addr addr, int port)
{
    static const char msg[]="checkps same host test packet\0";
    struct sockaddr_in dest;
    int sock;

    dest.sin_family=AF_INET;
    dest.sin_port=htons(port);
    dest.sin_addr=addr;

    if ((sock=socket(PF_INET, SOCK_DGRAM, 0))<0)
	return -1;
    sendto(sock, (void *) msg, sizeof(msg),
	   MSG_DONTROUTE | MSG_DONTWAIT | MSG_NOSIGNAL,
	   (struct sockaddr *) &dest, sizeof(dest));
    close(sock);
    return 1;
}

int is_localhost(const struct in_addr addr)
{
    int p, rfd, v, nrd;
    fd_set rd, wr, ex;
    struct timeval timout;
    char buf[256];
    struct sockaddr *src;
    socklen_t srclen;
    
    if ((p=recv_socket(10000, &rfd))==-1)
	return -1;
#ifdef TEST
    printf("Listening on UDP port %d, fd %d\n", p, rfd);
#endif /* TEST */

    if (send_dgram(addr, p)!=1)
    {
	close(rfd);
	return -1;
    }
    
    FD_ZERO(&wr);
    FD_ZERO(&ex);
    FD_ZERO(&rd);
    FD_SET(rfd, &rd);
    timout.tv_usec=0;
    timout.tv_sec=1;
    if ((v=select(rfd+1, &rd, &wr, &ex, &timout))!=1)
    {
	close(rfd);		/* Close listening socket */
	return v;
    }
    src=(struct sockaddr *) alloca(sizeof(struct sockaddr_in));
    srclen=sizeof(struct sockaddr_in);
    nrd=recvfrom(rfd, (void *) buf, sizeof(buf), 0, src, &srclen);
    close(rfd);			/* Close listening socket */
#ifdef TEST
    if (src->sa_family==AF_INET)
    {
	printf("Read %d bytes from %s\n", nrd,
	       inet_ntoa(((struct sockaddr_in *) src)->sin_addr));
    }
#endif /* TEST */
    return 1;
}
    
#ifdef TEST
int main(void)
{
    int r;
    char *t;
    struct in_addr addr;

    char line[256];
    while(1)
    {
	printf("IP>");
	if ((fgets(line, sizeof(line), stdin))==NULL)
	    return 0;

	t=(line[0]=='\0') ? line : line+strlen(line)-1;
	while (isspace(*t))
	{
	    *t='\0';
	    if (--t==line)
		break;
	}
	if (line[0]=='\0' || strcmp(line,"quit")==0)
	    return 0;
	
	if (!inet_aton(line, &addr))
	{
	    struct hostent *h;
	    int i;
	    if ((h=gethostbyname(line))==NULL)
	    {
		printf("%s not understood\n");
		continue;
	    }

	    printf("Official host name %s\n", h->h_name);
	    for (i=0; (*(h->h_aliases+i)!=NULL); i++)
	    {
		printf("aka %s\n", *(h->h_aliases+i));
	    }

	    if (h->h_addrtype!=AF_INET)
	    {
		printf("Address type not AF_INET\n");
		continue;
	    }
	    for (i=0; (*(h->h_addr_list+i)!=NULL); i++)
	    {
		printf("Address %s\n",
		       inet_ntoa(*((struct in_addr *) h->h_addr_list[i])));
	    }
	    addr=*((struct in_addr *) h->h_addr_list[0]);
	}
	switch(r=is_localhost(addr))
	{
	case -1:
	    printf("Test failed with error %s\n", strerror(errno));
	    break;

	case 0:
	    printf("Not the same host\n");
	    break;

	case 1:
	    printf("Is the local machine\n");
	    break;

	default:
	    printf("Unexpect return value %d\n", r);
	    break;
	}
    }
    /* NOT REACHED */
}
#endif /* TEST */

		
