/* $Id: process.c,v 1.5 2000/10/10 22:10:44 dps Exp $ */
/* Process analysis for linux */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */
#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif /* HAVE_STDLIB_H */
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif /* HAVE_ERRNO_H */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "linux/os-cfg.h"
#include "linux/proc.h"
#include "libdetail/detail.h"
#include "prog.h"

static int log_cmdline(int pid)
{
    char filename[FNAME_LEN];	/* File name */
    char filebuf[CMD_BYTES+1];	/* bytes of command line to report */
    int all_rd,s;		/* Read all of command? */
    FILE *f;

    if (pid==0)
	return 1;		/* Pid 0 does not fetaure in /proc */

    if (snprintf(filename, FNAME_LEN, "/proc/%d/cmdline", pid)>=FNAME_LEN)
	return 0;		/* Paranoia */
    if ((f=open_procfile(filename))==NULL)
	return 0;

    s=fread((void *) filebuf, sizeof(char), CMD_BYTES,  f);
    if (ferror(f) || s==0)
	return 0;
    all_rd=feof(f);
    fclose(f);
    filebuf[s]='\0';
    filebuf[CMD_BYTES]='\0';	/* Ensure termination */
    do_log ("pid %d: command `%s' %s", pid, filebuf,
	 (all_rd) ? "" : "(truncated)" );
    return 1;
}

static int getpids(int pid, int *pids, char *state)
{
    char filename[FNAME_LEN];	/* Filename buffer */
    char filebuf[STAT_BYTES];	/* Contents of file */
    FILE *f;
    char *s, *t;
    int n, npid;
    size_t sz;

    if (pid==0)			/* Pid 0 not featured in /proc */
	return 0;

    if (snprintf(filename, FNAME_LEN, "/proc/%d/stat", pid)>=FNAME_LEN)
	return 0;		/* Paranoia */


    if ((f=open_procfile(filename))==NULL)
    {
	do_log("Problem openning %s. This is very suspicious", filename);
	return 0;
    }

    sz=fread((void *) filebuf, sizeof(char), STAT_BYTES,  f);
    errno=0;
    if (ferror(f) || sz==0)
    {
	do_log("Pid %d: Could not read stat (error %s, size %d)", pid,
	    strerror(errno), sz);
	fclose(f);
	return 0;
    }
    fclose(f);

    /* We want ppid, pgrp, session */
    if ((s=strrchr(filebuf, '('))==NULL)
    {
	do_log("Pid %d: /proc/<pid>/stat format violation", pid);
	return 0;
    }
    if ((t=strrchr(filebuf, ')'))==NULL || t<s)
    {
	do_log("Pid %d: /proc/<pid>/stat format violation", pid);
	return 0;
    }
    n=sscanf(filebuf, "%d", &npid);	/* Get pid */
    if (pid!=npid)
    {
	do_log("Pid %d:*Very* suspect system---stat thinks pid id %d != %d", pid, npid, pid);
    }
    if ((t+2-filebuf)>=sz)
    {
	do_log("Pid %d: /proc/<pid>/stat format violation", pid);
	return 0;		/* Heavy paranoia */
    }
    n+=sscanf(t+2, "%c %d %d %d", state, pids, pids+1,
	    pids+2);
    if (n<5)
    {
	do_log("Pid %d: /proc/<pid>/stat format violation", pid);
	return 0;
    }    
    do_log("pid %d: pid %d state %c, ppid=%d pgrp=%d session=%d\n",
	    pid, npid, *state, pids[0], pids[1], pids[2]);
}

static int remdups(int *nums, int n)
{
    int i, j, min, midx;

    /* First sort the numbers using selection sort */
    for (i=0; (i<n); i++)
    {
	min=*(nums+i);
	midx=i;
	for (j=i+1; (j<n); j++)
	{
	    if (*(nums+j)<min)
	    {
		min=*(nums+j);
		midx=j;
	    }
	}
	if (midx!=i)
	{
	    *(nums+i) += *(nums+midx);
	    *(nums+midx)= *(nums+i) - *(nums+midx);
	    *(nums+i)= *(nums+i) - *(nums+midx);
	}
    }
    
    /* Next cull the duplicates */
    for (i=1, j=0; (i<n); i++)
    {
	if (*(nums+i)!=*(nums+j))
	    j++;
	*(nums+j)=*(nums+i);
    }
    return j+1;
}

int *os_read_stat(int pid)
{
    int *pids, n, i;
    char state;


    if ((pids=(int *) malloc(sizeof(int)*12))==NULL)
    {
	do_log("Could not allocate memory for pids");
	return NULL;
    }

    /* ppid, pgrp, session and thier relatives are interesting */
    if (!getpids(pid, pids, &state))
    {
	return NULL;
    }
    for (n=3, i=0; (i<3); i++)
    {
	if (getpids(pids[i], pids+n, &state))
	    n+=3;
    }
    n=remdups(pids, n);

    if (!log_cmdline(pid))
	do_log("pid %d: could not obtain command line\n", pid);
    for (i=0; i<n; i++)
    {
	if (!log_cmdline(pids[i]))
	    do_log("pid %d: could not obtain command line\n", pids[i]);
	fd_analysis(pids[i]);	/* View the files the process uses. This */
				/* hopefully gets the cracker's IP numbers. */
    }
    return pids;
}
